/* Application Explorer - a general-purpose application cataloger/indexer
 * Copyright (C) 2006  Aliasgar Lokhandwala <d7@freepgs.com> 
 * http://appexp.sourceforge.net/
 *
 * IBrwFile.java: the interface that describes application descriptors
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the 
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 */

package app.appexplorer.file;

import java.util.Vector;

/**
 * The IBrwFile interface defines an "application descriptor" file.
 * 
 * The default "application descriptor" used by application explorer is a .brw
 * file implemented by the BrwFile. But this interface can be implemented in
 * future to use PAD Files or any other file as a valid "application
 * descriptor".
 * 
 * The idea behind this interface is to give us independence from the physical
 * structure of an application descriptor file. Any file can be used as an
 * application descriptor if its class just implements this interface, it will
 * easily integrate with the rest of the app.
 * 
 * Note: you'll have to modify the BrwFileHunter to add support for any new
 * class that implements this interface. Also you'll need to write a search
 * function similar to BrwFileHunter.getBrwFilesList()
 * 
 * @author ali.
 * @version 2.0
 * @see app.appexplorer.file.BrwFile
 * @see app.appexplorer.hunt.BrwFileHunter
 * @see app.appexplorer.hunt.BrwFileHunter#getBrwFilesList()
 */
public interface IBrwFile {
    /**
     * Returns a string that specifies what category this app belongs to. (eg.
     * Browsers, Drivers, Music) Should not return null. If no category can be
     * found return "Default".
     * 
     * @return String - application category
     */
    public String getCategory();

    /**
     * Returns the name of this application. (eg. FireFox, Apache) Should not
     * return null. If no name can be found return "UnknownApplication"
     * 
     * @return String - application name
     */
    public String getName();

    /**
     * Returns the absolute path of this file. The path string can be the
     * canonical path or the abstract pathname.
     * 
     * @see java.io.File#getPath()
     * @return String - file path
     */
    public String getPath();

    /**
     * Returns the folder containing this application descriptor file. The path
     * string can be the canonical path or the abstract pathname. <br>
     * The containing folder is assumed to be the root folder for the described
     * application. If this is not true this function should actually return the
     * root folder for the described application.
     * 
     * @see java.io.File#getParent()
     * @return String - folder path
     */
    public String getContainingFolder();

    /**
     * Gets the short description for this application. Dont return null. if no
     * description is found return the long description or return "No
     * description available."
     * 
     * @return String - short description
     */
    public String getShortDescriptionText();

    /**
     * Gets the long description for this application. Dont return null. if no
     * description is found return the short description or return "No
     * description available."
     * 
     * @return String - short description
     */
    public String getLongDescriptionText();

    /**
     * Gets a vector containing filters used to search for setup files. <br>
     * eg {*.exe, *.tar.gz, readme.txt}
     * 
     * Cannot return null. The returned hash can be empty.
     * 
     * @return Vector - containing filters specified in the file
     * @see app.appexplorer.hunt.SetupFileHunter
     */
    public Vector getSetupFileFilters();

    /**
     * Specifies that the setup files for this application may be inside
     * subfolders of the folder containing the "application descriptor"
     * 
     * @return - true if subfolders must also be searched for setup files.
     */
    public boolean searchSubFoldersForSetupFiles();
}