/* Application Explorer - a general-purpose application cataloger/indexer
 * Copyright (C) 2006  Aliasgar Lokhandwala <d7@freepgs.com> 
 * http://appexp.sourceforge.net/
 *
 * IniFile.java: the class for reading .ini files
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the 
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 */

package app.appexplorer.file;

import java.io.*;
import java.util.*;
import java.util.logging.Level;
import java.util.logging.LogManager;
import java.util.logging.Logger;

/**
 * Implements a simple reader/parser for Ini files.
 * 
 * @author ali
 * @version 2.0
 */
public class IniFile implements IIniFile {
    /**
     * The file object attached to this class.
     */
    protected File m_file = null;

    private BufferedReader m_inBuffer = null;

    private static Logger m_globalLogger = LogManager.getLogManager()
            .getLogger("global");

    /**
     * Creates a IniFile object and attaches it to the supplied File.
     * 
     * @param file -
     *            The file object to attach to this object.
     */
    public IniFile(File file) {
        m_file = file;
    }

    /**
     * Closes the input read buffer and stream if created by readline.
     * readLine() calls this function internally to close the stream after it
     * finish reading the file.
     * 
     * @see IniFile#readLine()
     */
    private void reset() {
        if (m_inBuffer != null) {
            try {
                m_inBuffer.close();
                m_inBuffer = null;
            } catch (IOException e) {
                //ignore
                m_globalLogger.log(Level.SEVERE, "Exception closing file "
                        + e.getMessage());
            } finally {
                m_inBuffer = null;
            }
        }
    }

    /**
     * Reads next line from the file attached to this class. This function is
     * intended to be called from a while loop.
     * 
     * @return String - The line read from file, or null if EOF/Error.
     */
    protected String readLine() {
        try {
            if (m_inBuffer == null) {//Note: not thread safe!
                //create the read input buffer.
                m_inBuffer = new BufferedReader(new FileReader(m_file));
            }
        } catch (IOException ex1) {
            m_globalLogger.log(Level.SEVERE, "Exception creating read buffer "
                    + ex1.getMessage());
            m_globalLogger.log(Level.SEVERE,
                    "Check if we have read permissions on file "
                            + m_file.getAbsolutePath());

            reset();
            return null;
        }

        try {
            String line = m_inBuffer.readLine();
            if (line == null) {//EOF condition
                reset();
            }
            return line;
        } catch (IOException ex2) {
            m_globalLogger.log(Level.SEVERE, "Exception reading file "
                    + ex2.getMessage());

            reset();
            return null;
        }
    }

    /**
     * Checks to see if passed string is a section heading. Section headings are
     * strings in square brackets. eg. [MySection]
     * 
     * @param str -  
     *            The string to test. Should not be null.
     * @return Boolean - True if the string looks like a section heading, false
     *         otherwise.
     */
    private boolean isSection(String str) {
        if (str.matches(".*\\[.*\\].*")) {
            return true;
        }
        return false;
    }

    /**
     * Removes any comments from the supplied string. Ini files can contain
     * comments on any line. Comments start with a ;. Everything after the ; is
     * considered as a comment and removed.
     * 
     * @param s -
     *            The string from which comment is to be cleaned. Should not be
     *            null.
     * @return The string after cleaning off any comments. Will not return null.
     */
    private String cleanComment(String s) {
        int comm = s.indexOf(';');

        //there is no comment in this line
        if (comm == -1) {
            return s;
        }

        //the whole line is a comment, return a blank string
        if (comm == 0) {
            m_globalLogger.log(Level.FINE, "Line was ignored as comment " + s);
            return "";
        }

        //Remove the comment part and return remaning string.
        return s.substring(0, comm).trim();
    }

    /**
     * @inheritDoc Section names must be unique, otherwise only the first
     *             matching section will be returned.
     */
    public Hashtable readSection(String section) {
        Hashtable nv = new Hashtable();

        boolean found = false; //indicates if we found the section

        /*
         * Indicates the section has closed. A section closes when the next one
         * begins or the file ends.
         */
        boolean closed = false;

        //read a line fron the file
        String line = readLine();

        while (line != null && !closed) {
            //clean off any whitespace and comments
            line = line.trim();
            line = cleanComment(line);

            if (line.length() != 0) {//ignores any blank lines
                if (found) {
                    if (line.matches(".+=.+")) {
                        //Name=Value pair
                        String[] NameValue = line.split("=");
                        nv.put(NameValue[0].toLowerCase().trim(), NameValue[1]
                                .trim());
                    } else {
                        //Has another section started..?
                        if (isSection(line)) {
                            //yes, so close off our section
                            closed = true;
                        }
                    }
                } else/*not-found*/{
                    if (isSection(line)) {

                        /*
                         * Note: [MySection], [ MySection ], [MySection ] are
                         * all valid section names so we remove the brackets,
                         * then trim off any spaces and then compare the section
                         * names.
                         */
                        line = line.replace("[", "");
                        line = line.replace("]", "");
                        line = line.trim();

                        //see if the section name matches the supplied string
                        if (line.equalsIgnoreCase(section)) {
                            //yippe! we found the section we were looking for.
                            found = true;
                            m_globalLogger.log(Level.FINE, "Section found: "
                                    + section);
                        }
                    }
                }

            }/*end if(line.length()!=0)*/

            //Read next line
            line = readLine();
        }/*end-while*/

        //close off the file
        reset();

        return nv;
    }
}