/* Application Explorer - a general-purpose application cataloger/indexer
 * Copyright (C) 2006  Aliasgar Lokhandwala <d7@freepgs.com> 
 * http://appexp.sourceforge.net/
 *
 * BrwFileHunter.java: The class that helps in searching for 
 * 	application descriptors
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the 
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 */

package app.appexplorer.hunt;

import java.io.File;
import java.util.Hashtable;
import java.util.Vector;
import java.util.logging.Level;
import java.util.logging.LogManager;
import java.util.logging.Logger;

import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;

import app.appexplorer.file.BrwFile;
import app.appexplorer.file.IBrwFile;
import app.appexplorer.ui.IStatusBox;

/**
 * Hunts for "application descriptor" files.
 * 
 * Note: When a new application descriptior is added to this app, this class
 * will need to be modified to hunt for it. Specifically you'll need to write a
 * new function similar to getBrwFilesList() and modify getBrwModel()
 * 
 * @version 2.0
 * @author ali
 */
public final class BrwFileHunter {

    /**
     * The folder to start at. Default is the current folder.
     */
    private String m_sBaseDir = ".";

    /**
     * If this variable is set, search logs will be directed to this box.
     */
    private IStatusBox m_status = null;

    /**
     * Used for logging messages.
     */
    private static Logger m_globalLogger = LogManager.getLogManager()
            .getLogger("global");

    /**
     * Create a hunter starting at the given base folder.
     * 
     * @param sBaseDir -
     *            represents the folder to start search at. Must not be null.
     *            Must represent a folder/drive path.
     */
    public BrwFileHunter(String sBaseDir) {
        m_sBaseDir = sBaseDir;
        m_globalLogger.log(Level.INFO, "Searching for brw files in: "
                + sBaseDir);
    }

    /**
     * Set the box to use for logging search progress messages. Can be used to
     * display information to the user for long searches.
     * 
     * @param s -
     *            the status box to use for logging informative messages.
     */
    public void setStatusBox(IStatusBox s) {
        m_status = s;
    }

    /**
     * Constructs a tree model using information from found "application
     * descriptor" files. With minimum modification this function can handle new
     * "application descriptors"
     * 
     * @return A treemodel reprsenting the hierarchy of found applications.
     *         Note: The root node and category nodes of this model are strings,
     *         but leaf nodes are IBrwFile Objects.
     */
    public DefaultTreeModel getBrwModel() {
        DefaultTreeModel Tree = null;

        //there is just one root node.
        DefaultMutableTreeNode root = null;

        Vector filelist = null;

        /*
         * Append all "application descriptors" into the filelist here.
         * No other modification to this function should be required.
         * 
         * eg. filelist.addAll(get <MyDescriptor>FilesList());
         * eg. filelist.addAll(getPADFilesList());
         */
        filelist = getBrwFilesList();

        
        //Note: The root node and category nodes are strings, but leaf nodes
        //are IBrwFile Objects
        if (filelist.size() == 0) {
            Tree = new DefaultTreeModel(root = new DefaultMutableTreeNode(
                    "No Applications Found!"));
        } else {
            Tree = new DefaultTreeModel(root = new DefaultMutableTreeNode(
                    "Applications"));
        }

        /*
         * This Hashtable contains DefaultMutableTreeNode objects for values and
         * category strings as keys
         */
        Hashtable categories = new Hashtable();

        IBrwFile file = null;
        DefaultMutableTreeNode current = null;
        for (int i = 0; i < filelist.size(); i++) {
            file = (IBrwFile) filelist.get(i);

            //check if this category already exists
            current = (DefaultMutableTreeNode) categories.get(file
                    .getCategory());
            if (current == null) {
                //add new category
                root.add(current = new DefaultMutableTreeNode(file
                        .getCategory()));
                
                //add this new node to the catogory hash
                categories.put(file.getCategory(), current);
            }
            
            //add this application under existing category
            current.add(new DefaultMutableTreeNode(file));
        }

        return Tree;
    }

    /**
     * Searches for .brw files in the base dir and returns a vector containing
     * BrwFile objects representing found files.
     * 
     * To create new "application descriptors" create methods similar to this
     * one, remember its not really important how or what you search for. Just
     * return a Vector containing IBrwFile objects from your function.
     * 
     * @return Vector containing IBrwFile objects representing the found files.
     * @see FolderSearch
     * @see BrwFile
     * @see IBrwFile
     */
    private Vector getBrwFilesList() {
        FolderSearch s = new FolderSearch("*.brw", m_sBaseDir);
        s.setStatusBox(m_status);

        s.doSearch(true);

        //convert the File list into the correct parser class
        IBrwFile file = null;
        
        Vector filelist = s.getFiles(); //contains File objects
        
        //will contain BrwFile objects attached to IBrwFile
        Vector newList = new Vector();

        //loop through the vector and convert to correct IBrwType implementing type.
        for (int i = 0; i < filelist.size(); i++) {
            file = new BrwFile((File) filelist.get(i));
            newList.add(file);
        }

        return newList;
    }
}