/* Application Explorer - a general-purpose application cataloger/indexer
 * Copyright (C) 2006  Aliasgar Lokhandwala <d7@freepgs.com> 
 * http://appexp.sourceforge.net/
 *
 * FolderSearch.java: The class that helps in searching folders for files 
 * 	(based on given filter) 
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the 
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 */

package app.appexplorer.hunt;

import java.io.File;
import java.util.Vector;
import java.util.logging.Level;
import java.util.logging.LogManager;
import java.util.logging.Logger;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import app.appexplorer.ui.IStatusBox;

/**
 * FolderSearch incorporates basic file search features. FolderSearch searches
 * folders for subfolders and files. It can also do this recursively. Only files
 * matching the specified filter are searched. Folders cannot be filtered using
 * this class. All folders are always searched.
 * 
 * Usage: <br>
 * Startoff by creating a FolderSearch object. <br>
 * Call doSearch(). <br>
 * Call getFiles() to get the list of files found. <br>
 * Call getFolders() to get the list of folders found.
 * 
 * To do another search, do not reuse this object. Start fresh by creating
 * another FolderSearch object.
 * 
 * @author ali
 * @version 2.0
 */

public final class FolderSearch {
    /**
     * The filter pattern to use to filter files. Folders are not filtered by
     * this.
     */
    private Pattern m_sfilter;

    /**
     * The path at which to start searching, default is .
     */
    private String m_sRootPath = ".";

    /**
     * Vector to store files found
     */
    private Vector m_vtFiles = new Vector();

    /**
     * Vector to store folders found
     */
    private Vector m_vtFolders = new Vector();

    /**
     * Optional UI box that can show search status
     */
    private static IStatusBox m_status = null;

    /**
     * Used for logginf messages
     */
    private static Logger m_globalLogger = LogManager.getLogManager()
            .getLogger("global");

    /**
     * The default FolderSearch capable of searching for all files (*) in the
     * current (.) Folder. Call doSearch to actually perform the search.
     * 
     * @see FolderSearch#doSearch(boolean)
     */
    public FolderSearch() {
        m_sfilter = Pattern.compile(".*");
    }

    /**
     * Creates an object capable of searching for all files in specified folder.
     * Call doSearch to actually perform the search.
     * 
     * @param sRootPath -
     *            Path at which to start search.
     * 
     * @see FolderSearch#doSearch(boolean)
     */
    public FolderSearch(String sRootPath) {
        m_sfilter = Pattern.compile(".*");
        m_sRootPath = sRootPath;
    }

    /**
     * Creates an object capable of searching for files matching the supplied
     * filter in specified folder. Call doSearch to actually perform the search.
     * 
     * @param sFilter -
     *            A string representing the filesnames to select. This is a
     *            simnple DOS like file filter. * matches all and ? matches
     *            exactly one. Note: *.* matches all files that have any name
     *            and any entension. Files without extensions will not match
     *            *.*. To match all files just specify a *. Folders are not
     *            filtered.
     * @param sPath -
     *            The root folder to start searching at.
     * 
     * @see FolderSearch#doSearch(boolean)
     */
    public FolderSearch(String sFilter, String sPath) {
        /*
         * Convert the string DOS like file pattern to a reg-exp pattern
         */
        m_sfilter = Pattern.compile(sFilter.replaceAll("\\.", "\\\\.")
                .replaceAll("\\*", ".*").replaceAll("\\?", ".?"));
        m_sRootPath = sPath;
    }

    /**
     * This constructor is internally used to do recursive calls, to search in
     * subdirs. It avoids compiling the file pattern again.
     * 
     * @param pFilter -
     *            A reg-exp Pattern denoting the filenames to match.
     * @param sPath -
     *            The root folder to start searching at.
     */
    private FolderSearch(Pattern pFilter, String sPath) {
        m_sfilter = pFilter;
        m_sRootPath = sPath;
    }

    /**
     * Returns a Vector containing File objects where isFile() is true. Call
     * this after you call doSearch.
     * 
     * @return - Vector containing File objects. May be empty. Never null.
     * @see File#isFile()
     */
    public Vector getFiles() {
        return m_vtFiles;
    }

    /**
     * Convenience method that returns the first file found. Useful if the
     * search if expected to return only one file.
     * 
     * @return - The first file found. Returns null if no files were found or
     *         the doSearch method has not yet been called.
     */
    public File getFile() {
        if (m_vtFiles != null && m_vtFiles.size() > 0)
            return (File) m_vtFiles.elementAt(0);

        return null;
    }

    /**
     * Returns a Vector containing File objects where isDirectory() is true.
     * Call this after you call doSearch.
     * 
     * @return - Vector containing File objects. May be empty. Never null.
     * @see File#isDirectory()
     */
    public Vector getFolders() {
        return m_vtFolders;
    }

    /**
     * Used internally to remove non-matching files from the vectors. The search
     * always returns all files. This function is then called to remove files
     * that dont match the filter pattern.
     * 
     * @param filelist -
     *            Vector containing File objects to filter. Should not be null.
     *            Should not contain null elements.
     * @return - Filtered vector containing File objects that match the pattern.
     *         May be empty. Never null.
     */
    private Vector Filter(Vector filelist) {
        Vector dest = new Vector();
        //loop through the filelist
        for (int i = 0; i < filelist.size(); i++) {
            //see if each filename matches the pattern
            File f = (File) filelist.get(i);
            Matcher m = m_sfilter.matcher(f.getName().toLowerCase());
            //if it matches keep it
            if (m.matches()) {
                dest.add(f);
                //System.out.println("Match success: "+f.getName());
            } else {
                //System.out.println("Match failed: "+f.getName());
            }
        }

        return dest;
    }

    /**
     * Convenience method that doesnt search subfolders.
     * 
     * @see FolderSearch#doSearch(boolean)
     */
    public void doSearch() {
        doSearch(false);
    }

    /**
     * Call this method only once to actually search for files. Do not call this
     * method again. Make a new FolderSearch object to perform a fresh search.
     * 
     * @param bSearchSubDirs -
     *            if true, any folders inside the root folder are also searched
     *            for files.
     * 
     * TODO This method will be more efficient if made multi-threaded. Need to
     * work on that.
     */
    public void doSearch(boolean bSearchSubDirs) {
        //Start at given root
        File search = new File(m_sRootPath);

        //root must be a folder
        if (!search.isDirectory()) {
            m_globalLogger.log(Level.SEVERE, search.getAbsolutePath()
                    + " is not a directory!");
            return;
        }

        //update any UI status box, and logs.
        showStatus("Searching: " + search.getAbsolutePath());
        m_globalLogger
                .log(Level.FINER, "Searching " + search.getAbsolutePath());

        //get all files and folders in folder
        File[] FoundFiles = search.listFiles();

        /*
         * May return null if search is a file but was wrongly treated as a
         * Directory by isDirectory(), or an IO error occured
         */
        if (FoundFiles == null) {
            m_globalLogger.log(Level.WARNING, "Can't read, skipped "
                    + search.getAbsolutePath());
        }

        //sort them into buckets "files" and "folders"
        for (int i = 0; (FoundFiles != null) && (i < FoundFiles.length); i++) {
            if (FoundFiles[i].isFile() && FoundFiles[i].canRead()) {
                m_vtFiles.add(FoundFiles[i]);
            }

            if (FoundFiles[i].isDirectory()) {
                if (FoundFiles[i].canRead()) {
                    m_vtFolders.add(FoundFiles[i]);

                    //Go depth first into the folder
                    if (bSearchSubDirs) {
                        FolderSearch subfolder = new FolderSearch(m_sfilter,
                                FoundFiles[i].getAbsolutePath());
                        subfolder.doSearch(true);

                        //add subfolder contents to our array
                        m_vtFiles.addAll(subfolder.getFiles());
                        m_vtFolders.addAll(subfolder.getFolders());
                    }

                } else /* !.canRead() */{
                    m_globalLogger.log(Level.WARNING, "Can't read, skipped "
                            + FoundFiles[i].getAbsolutePath());
                }
            }
        }

        //finally, filter off unwanted files
        m_vtFiles = Filter(m_vtFiles);
    }

    /**
     * Specify a UI component capable of displaying status messgages.
     * 
     * @param s -
     *            A UI component capable of displaying status. Provide null to
     *            disable.
     */
    public void setStatusBox(IStatusBox s) {
        m_status = s;
    }

    /**
     * Get the current UI component used for displaying status messgages.
     * 
     * @return - The UI component set earlier using setStatusBox or null if
     *         disabled.
     */
    public IStatusBox getStatusBox() {
        return m_status;
    }

    /**
     * Used internally to update status if UI status display is enabled.
     * 
     * @param sStatus -
     *            The status message string to show in the status box.
     */
    private void showStatus(String sStatus) {
        if (m_status != null) {
            m_status.setText(sStatus);
        }
    }
}