/* Application Explorer - a general-purpose application cataloger/indexer
 * Copyright (C) 2006  Aliasgar Lokhandwala <d7@freepgs.com> 
 * http://appexp.sourceforge.net/
 *
 * GlobalUtils.java: Provides static utility methods.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the 
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 */

package app.appexplorer.util;

import java.awt.Toolkit;
import java.awt.datatransfer.StringSelection;
import java.io.File;
import java.io.IOException;
import java.util.logging.Level;
import java.util.logging.LogManager;
import java.util.logging.Logger;

/**
 * Provides static utility methods.
 * 
 * @author ali
 * @version 2.0
 */
public final class GlobalUtils {

    /**
     * Logger used for logging messages.
     */
    private static Logger m_globalLogger = LogManager.getLogManager()
            .getLogger("global");

    /*
     * Big OS Categories
     */
    public final static int OS_UNKNOWN = 0;

    public final static int OS_WIN = 1; //all of microsofts kittens.

    public final static int OS_NIX = 2; //linux, unix

    public final static int OS_APPLE = 3; //Mac OS

    /**
     * Get the big category of this OS. This function uses the name of the os
     * returned by the Java VM and tries to classify it into one of the Big
     * categories.
     * 
     * This function is critical for other OS specific functions.
     */
    public static int getBigOsType() {
        String name = System.getProperty("os.name");
        name = name.toLowerCase();

        if (name.indexOf("window") != -1) {
            return OS_WIN;
        }

        m_globalLogger.log(Level.SEVERE,
                "Failed to detect OS type. OS specific calls may not work!");
        m_globalLogger.log(Level.INFO,
                "----------------------------------------");
        m_globalLogger.log(Level.INFO,
                "Your OS may not be currently supported by this program.");
        m_globalLogger.log(Level.INFO,
                "We'll be glad to add support for your OS.");
        m_globalLogger.log(Level.INFO,
                "Do send us your request with this log file.");
        m_globalLogger.log(Level.INFO, "OS Name: "
                + System.getProperty("os.name"));
        m_globalLogger.log(Level.INFO, "OS Architecture: "
                + System.getProperty("os.arch"));
        m_globalLogger.log(Level.INFO, "OS Version: "
                + System.getProperty("os.version"));
        m_globalLogger.log(Level.INFO,
                "----------------------------------------");

        return OS_UNKNOWN;
    }

    /**
     * Tries to convert the given abstract path name to its OS specific
     * (canonical) form. If there is a faliure/error in conversion, returns
     * sPath as-is. So there is no gurantee that the returned path string will
     * be the canonical form. Main use is only for logging or showing path
     * information to the user.
     * 
     * @param sPath -
     *            The abstract path. Should not be null.
     * @return - The canonical path, or sPath if error occurs.
     */
    public static String toCanonicalPath(String sPath) {
        File f = new File(sPath);
        try {
            return f.getCanonicalPath();
        } catch (IOException e) {
            m_globalLogger.log(Level.SEVERE, "Failed to get canonical path "
                    + e.getMessage());
            return sPath;
        }
    }

    /**
     * Convenience method that copies the supplied string to the system
     * clipboard.
     * 
     * @param str -
     *            The String to copy to the system clipboard.
     * 
     * @see StringSelection
     */
    public static void copyToSysClipboard(String str) {
        copyToSysClipboard(new StringSelection(str));
    }

    /**
     * Copies the supplied string selection to system clipboard.
     * 
     * @param strSel -
     *            The String to copy to the system clipboard.
     * 
     * @see StringSelection
     * @see GlobalUtils#copyToSysClipboard(String)
     */
    public static void copyToSysClipboard(StringSelection strSel) {
        try {
            Toolkit.getDefaultToolkit().getSystemClipboard().setContents(
                    strSel, strSel);
        } catch (Exception e) {
            m_globalLogger.log(Level.SEVERE, "Failed to copy string " + strSel
                    + " to the clipboard " + e.getMessage());
        }
    }

    /**
     * Convert the supplied value in bytes to KBytes.
     * 
     * @param nBytes -
     *            Amount in Bytes.
     * @return - Amount in KBytes.
     */
    public static long toKBytes(long nBytes) {
        nBytes = nBytes / 1024;
        return nBytes;
    }

    /**
     * Tries to parse the supplied string into an int. Returns def on failure.
     * 
     * @param val -
     *            The String value to parse. Can be null.
     * @param def -
     *            The default value if parse fails.
     * @return - The parsed value of val, or def if error occurs.
     */
    public static int nvlInt(Object val, int def) {
        int retVal = def;
        try {
            retVal = Integer.parseInt((String) val);
        } catch (Exception e) {
            //ignore
        }
        return retVal;
    }

    /**
     * Tries to parse the supplied string into an Log Level. Returns def on
     * failure.
     * 
     * @param val -
     *            The String value to parse. Can be null.
     * @param def -
     *            The default value if parse fails.
     * @return - The parsed value of val, or def if error occurs.
     * 
     * @see Level
     */
    public static Level nvlLogLevel(Object val, Level def) {
        Level retVal = def;
        try {
            retVal = Level.parse(((String) val).toUpperCase());
        } catch (Exception e) {
            //ignore
        }
        return retVal;
    }

    /**
     * Tries to parse the supplied string into an boolean. Returns def on
     * failure.
     * 
     * @param val -
     *            The String value to parse. Can be null.
     * @param def -
     *            The default value if parse fails.
     * @return - The parsed value of val, or def if error occurs.
     */
    public static boolean nvlBoolean(Object val, boolean def) {
        boolean retVal = def;

        if (val == null)
            return def;

        try {
            retVal = Boolean.parseBoolean((String) val);
        } catch (Exception e) {
            //ignore
        }
        return retVal;
    }

    /**
     * Tries to parse the supplied string into an String. Returns def on
     * failure.
     * 
     * @param val -
     *            The String value to parse. Can be null.
     * @param def -
     *            The default value if parse fails.
     * @return - The parsed value of val, or def if error occurs.
     */
    public static String nvlString(Object val, String def) {
        try {
            String sVal=(String) val;
            if (sVal != null && sVal.length()!=0)
                return sVal;
        } catch (Exception e) {
            return def;
        }

        return def;
    }

    /**
     * Contains os specific functions. Detects the current os and executes
     * commands for it.
     * 
     * @version 2.0
     * @author ali
     */
    public static final class os {

        /**
         * Open the folder given by the supplied path string in a OS specific
         * way. The string should be the absolute canonical path to a folder.
         * 
         * It is not recommended to call this function directly. Use
         * openFolder() instead which is more flexible and less error prone.
         * 
         * @see java.io.File#getCanonicalPath()
         * @param sPath -
         *            Path of the folder to open. Should not be null.
         * @return boolean indicating success/faliure of the operation.
         */
        public static boolean openFolderSys(String sPath) {
            m_globalLogger.log(Level.FINE, "Opening folder " + sPath);

            switch (getBigOsType()) {
            case OS_WIN:
                return openWinFolder(sPath);
            case OS_UNKNOWN:
                return false;
            }
            return false;
        }

        /**
         * Open the folder given by the supplied path string. The string can
         * represent the relative and abstract path. If sPath represents a file,
         * the folder containing it will be opened.
         * 
         * @param sPath -
         *            Path of the folder to open. Should not be null.
         * @return boolean indicating success/faliure of the operation.
         */
        public static boolean openFolder(String sPath) {
            File f = new File(sPath);

            if (!f.isAbsolute()) {
                //make the path absolute
                f = f.getAbsoluteFile();
            }

            if (f.isFile()) {
                //try to get the parent folder
                f = f.getParentFile();
                if (f == null) {
                    m_globalLogger.log(Level.SEVERE,
                            "Failed to get the parent folder for the file "
                                    + sPath);
                    return false;
                }
            }

            try {
                return openFolderSys(f.getCanonicalPath());
            } catch (IOException e) {
                m_globalLogger.log(Level.SEVERE,
                        "Failed to get canonical path " + e.getMessage());
                return false;
            }
        }

        /**
         * Implementation of openFolder for Windows. Uses the windows ui shell
         * "explorer.exe" to open a folder.
         * 
         * @param sPath -
         *            the absolute canonical path of the folder to open.
         * @return - true on success.
         */
        private static boolean openWinFolder(String sPath) {
            try {

                Process p = Runtime.getRuntime().exec(
                        new String[] { "explorer.exe", sPath });
                if (p != null)
                    return true;

            } catch (Exception e) {
                m_globalLogger.log(Level.SEVERE, "Failed to open folder "
                        + e.getMessage());
                return false;
            }

            return false;
        }

        /**
         * Open the file given by the supplied path string in an OS specific
         * way. The string should be the absolute canonical path to a file.
         * 
         * Note that the term "open" may stand for Edit, or Excecute or View
         * depending on the OS we are on and the file being "opened"
         * 
         * It is not recommended to call this function directly. Use openFile()
         * instead which is more flexible and less error prone.
         * 
         * @see java.io.File#getCanonicalPath()
         * @param sPath -
         *            Path of the file to open. Should not be null.
         * @return boolean indicating success/faliure of the operation.
         */
        public static boolean openFileSys(String sPath) {
            m_globalLogger.log(Level.FINE, "Opening file " + sPath);

            switch (getBigOsType()) {
            case OS_WIN:
                return openWinFile(sPath);
            case OS_UNKNOWN:
                return openGenericFile(sPath);
            }
            return false;
        }

        /**
         * Open the file given by the supplied path string. The string can be
         * the relative abstract path.
         * 
         * @param sPath -
         *            Path of the file to open. Should not be null.
         * @return boolean indicating success/faliure of the operation.
         */
        public static boolean openFile(String sPath) {
            File f = new File(sPath);

            if (!f.isFile()) {
                m_globalLogger.log(Level.SEVERE, "Not a file " + sPath);
                return false;
            }

            if (!f.isAbsolute()) {
                //make the path absolute
                f = f.getAbsoluteFile();
            }

            try {
                return openFileSys(f.getCanonicalPath());
            } catch (IOException e) {
                m_globalLogger.log(Level.SEVERE,
                        "Failed to get canonical path " + e.getMessage());
                return false;
            }
        }

        /**
         * Implementation of openFile in a generic OS independent way. This
         * operation will not succeed if the sFile is not executable on the
         * current platform. So it can only be used to open executable files for
         * the platform, and should only be used if an OS Specific openFile is
         * not available.
         * 
         * @param sPath -
         *            the absolute canonical path of the file to open.
         * @see File#getCanonicalPath()
         * @return - true on success.
         */
        private static boolean openGenericFile(String sFile) {
            try {
                Process p = Runtime.getRuntime().exec(sFile);
                if (p != null)
                    return true;

            } catch (Exception e) {
                m_globalLogger.log(Level.SEVERE, "Failed to open file "
                        + e.getMessage());
                return false;
            }
            return false;
        }

        /**
         * Implementation of openFile for Windows. Uses the windows launcher
         * "start.exe" to open any file using its associated application.
         * 
         * @param sPath -
         *            the absolute canonical path of the file to open.
         * @see File#getCanonicalPath()
         * @return - true on success.
         */
        private static boolean openWinFile(String sFile) {
            try {
                Process p = Runtime.getRuntime().exec(
                        new String[] { "start.exe", sFile });
                if (p != null)
                    return true;

            } catch (Exception e) {
                m_globalLogger.log(Level.SEVERE, "Failed to open file "
                        + e.getMessage());
                return false;
            }
            return false;
        }
    }
}